@@ -37,6 +37,9 @@ in this Software without prior written authorization from The Open Group.
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
