@@ -1,5 +1,5 @@
 /* $XFree86: xc/programs/Xserver/hw/xfree86/os-support/sunos/sun_mouse.c,v 1.4 2002/01/25 21:56:21 tsi Exp $ */
-/* $XdotOrg: xc/programs/Xserver/hw/xfree86/os-support/sunos/sun_mouse.c,v 1.2 2004/04/23 19:54:13 eich Exp $ */
+/* $XdotOrg: xc/programs/Xserver/hw/xfree86/os-support/sunos/sun_mouse.c,v 1.8 2005/07/28 02:38:23 alanc Exp $ */
 /*
  * Copyright 1999-2001 The XFree86 Project, Inc.  All Rights Reserved.
  *
@@ -25,7 +25,7 @@
  * dealings in this Software without prior written authorization from the
  * XFree86 Project.
  */
-/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.
+/* Copyright 2004-2005 Sun Microsystems, Inc.  All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the
@@ -53,6 +53,10 @@
  * of the copyright holder.
  */
 
+#ifdef HAVE_XORG_CONFIG_H
+#include <xorg-config.h>
+#endif
+
 #include "xf86.h"
 #include "xf86_OSlib.h"
 #include "xf86OSmouse.h"
@@ -73,6 +77,14 @@
 # include <sys/vuid_wheel.h>
 #endif
 
+/* Support for scaling absolute coordinates to screen size in 
+ * Solaris 10 updates and beyond */
+#if !defined(HAVE_ABSOLUTE_MOUSE_SCALING)
+# ifdef MSIOSRESOLUTION /* Defined in msio.h if scaling support present */
+#  define HAVE_ABSOLUTE_MOUSE_SCALING
+# endif
+#endif
+
 /* Names of protocols that are handled internally here. */
 
 static const char *internalNames[] = {
@@ -83,22 +95,51 @@ static const char *internalNames[] = {
 static const char *solarisMouseDevs[] = {
     /* Device file:	Protocol: 			*/
     "/dev/mouse",	"VUID",		/* USB or SPARC */
-#ifdef __i386
+#if defined(__i386) || defined(__x86)
     "/dev/kdmouse",	"PS/2",		/* PS/2 */
 #endif
     NULL
 };
 
 typedef struct _VuidMseRec {
-    Firm_event event;
-    unsigned char *buffer;
-    char *strmod;
+    struct _VuidMseRec *next;    
+    InputInfoPtr	pInfo;
+    Firm_event 		event;
+    unsigned char *	buffer;
+    char *		strmod;
+    Bool(*wrapped_device_control)(DeviceIntPtr device, int what);
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING
+    Ms_screen_resolution	 absres;
+#endif
 } VuidMseRec, *VuidMsePtr;
 
+static VuidMsePtr	vuidMouseList = NULL;
 
 static int  vuidMouseProc(DeviceIntPtr pPointer, int what);
 static void vuidReadInput(InputInfoPtr pInfo);
 
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING
+static void vuidMouseSendScreenSize(ScreenPtr pScreen, VuidMsePtr pVuidMse);
+static void vuidMouseAdjustFrame(int index, int x, int y, int flags);
+
+static int vuidMouseGeneration = 0;
+static int vuidMouseScreenIndex;
+#define vuidMouseScreenPrivate(s) ((s)->devPrivates[vuidMouseScreenIndex].ptr)
+#endif /* HAVE_ABSOLUTE_MOUSE_SCALING */
+
+static inline
+VuidMsePtr getVuidMsePriv(InputInfoPtr pInfo)
+{
+    VuidMsePtr m = vuidMouseList;
+
+    while ((m != NULL) && (m->pInfo != pInfo)) {
+	m = m->next;
+    }
+
+    return m;
+}
+
+
 /*
  * Initialize and enable the mouse wheel, if present.
  *
@@ -111,22 +152,26 @@ vuidMouseWheelInit(InputInfoPtr pInfo)
 #ifdef HAVE_VUID_WHEEL
     wheel_state wstate;
     int nwheel = -1;
+    int i;
 
     wstate.vers = VUID_WHEEL_STATE_VERS;
     wstate.id = 0;
     wstate.stateflags = -1;
 
-    if (ioctl(pInfo->fd, VUIDGWHEELCOUNT, &nwheel) != 0)
+    SYSCALL(i = ioctl(pInfo->fd, VUIDGWHEELCOUNT, &nwheel));
+    if (i != 0)
 	return (0);
 
-    if (ioctl(pInfo->fd, VUIDGWHEELSTATE, &wstate) != 0) {
+    SYSCALL(i = ioctl(pInfo->fd, VUIDGWHEELSTATE, &wstate));
+    if (i != 0) {
 	xf86Msg(X_WARNING, "%s: couldn't get wheel state\n", pInfo->name);
 	return (0);
     }
 
     wstate.stateflags |= VUID_WHEEL_STATE_ENABLED;
 
-    if (ioctl(pInfo->fd, VUIDSWHEELSTATE, &wstate) != 0) {
+    SYSCALL(i = ioctl(pInfo->fd, VUIDSWHEELSTATE, &wstate));
+    if (i != 0) {
 	xf86Msg(X_WARNING, "%s: couldn't enable wheel\n", pInfo->name);
 	return (0);
     }
@@ -144,9 +189,9 @@ vuidPreInit(InputInfoPtr pInfo, const char *protocol, int flags)
 {
     MouseDevPtr pMse = pInfo->private;
     VuidMsePtr pVuidMse;
-    int buttons;
+    int buttons, i;
 
-    pVuidMse = xalloc(sizeof(VuidMseRec));
+    pVuidMse = xcalloc(sizeof(VuidMseRec), 1);
     if (pVuidMse == NULL) {
 	xf86Msg(X_ERROR, "%s: cannot allocate VuidMouseRec\n", pInfo->name);
 	xfree(pMse);
@@ -176,53 +221,98 @@ vuidPreInit(InputInfoPtr pInfo, const char *protocol, int flags)
     pVuidMse->buffer = (unsigned char *)&pVuidMse->event;
 
     pVuidMse->strmod = xf86SetStrOption(pInfo->options, "StreamsModule", NULL);
-    if (pVuidMse->strmod && 
-      	(ioctl(pInfo->fd, I_PUSH, pVuidMse->strmod) == -1)) {
-	xf86Msg(X_ERROR,
-	  "%s: cannot push module '%s' onto mouse device: %s\n",
-	  pInfo->name, pVuidMse->strmod, strerror(errno));
-	xf86CloseSerial(pInfo->fd);
-	pInfo->fd = -1;
-	xfree(pVuidMse);
-	xfree(pMse);
-	return FALSE;
+    if (pVuidMse->strmod) {
+	SYSCALL(i = ioctl(pInfo->fd, I_PUSH, pVuidMse->strmod));
+ 	if (i < 0) {
+	    xf86Msg(X_ERROR,
+		    "%s: cannot push module '%s' onto mouse device: %s\n",
+		    pInfo->name, pVuidMse->strmod, strerror(errno));
+	    xf86CloseSerial(pInfo->fd);
+	    pInfo->fd = -1;
+	    xfree(pVuidMse->strmod);
+	    xfree(pVuidMse);
+	    xfree(pMse);
+	    return FALSE;
+	}
     }
 
     buttons = xf86SetIntOption(pInfo->options, "Buttons", 0);
     if (buttons == 0) {
-	if(ioctl(pInfo->fd, MSIOBUTTONS, &buttons) == 0) {
+	SYSCALL(i = ioctl(pInfo->fd, MSIOBUTTONS, &buttons));
+	if (i == 0) {
 	    pInfo->conf_idev->commonOptions =
-	      xf86ReplaceIntOption(pInfo->conf_idev->commonOptions, 
-		"Buttons", buttons);
+		xf86ReplaceIntOption(pInfo->conf_idev->commonOptions, 
+				     "Buttons", buttons);
 	    xf86Msg(X_INFO, "%s: Setting Buttons option to \"%d\"\n",
-	      pInfo->name, buttons);
+		    pInfo->name, buttons);
 	}
     }
 
-    if (pVuidMse->strmod && 
-      (ioctl(pInfo->fd, I_POP, pVuidMse->strmod) == -1)) {
-	xf86Msg(X_WARNING,
-	  "%s: cannot pop module '%s' off mouse device: %s\n",
-	  pInfo->name, pVuidMse->strmod, strerror(errno));
+    if (pVuidMse->strmod) { 
+	SYSCALL(i = ioctl(pInfo->fd, I_POP, pVuidMse->strmod));
+	if (i == -1) {
+	    xf86Msg(X_WARNING,
+		    "%s: cannot pop module '%s' off mouse device: %s\n",
+		    pInfo->name, pVuidMse->strmod, strerror(errno));
+	}
     }
 
     xf86CloseSerial(pInfo->fd);
     pInfo->fd = -1;
 
-    /* Private structure */
-    pMse->mousePriv = pVuidMse;
-
     /* Process common mouse options (like Emulate3Buttons, etc). */
     pMse->CommonOptions(pInfo);
 
     /* Setup the local procs. */
+    pVuidMse->wrapped_device_control = pInfo->device_control;
     pInfo->device_control = vuidMouseProc;
     pInfo->read_input = vuidReadInput;
 
+    pMse->xisbscale = sizeof(Firm_event);
+
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING    
+    pVuidMse->absres.height = pVuidMse->absres.width = 0;
+#endif
+    pVuidMse->pInfo = pInfo;
+    pVuidMse->next = vuidMouseList; 
+    vuidMouseList = pVuidMse;
+
     pInfo->flags |= XI86_CONFIGURED;
     return TRUE;
 }
 
+static void
+vuidFlushAbsEvents(InputInfoPtr pInfo, int absX, int absY, 
+		   Bool *absXset, Bool *absYset)
+{
+#ifdef DEBUG
+    ErrorF("vuidFlushAbsEvents: %d,%d (set: %d, %d)\n", absX, absY, 
+	   *absXset, *absYset);
+#endif
+    if ((*absXset) && (*absYset)) {
+	xf86PostMotionEvent(pInfo->dev, 
+			    /* is_absolute: */    TRUE,
+			    /* first_valuator: */ 0,
+			    /* num_valuators: */  2,
+			    absX, absY);
+    } else if (*absXset) {
+	xf86PostMotionEvent(pInfo->dev, 
+			    /* is_absolute: */    TRUE,
+			    /* first_valuator: */ 0,
+			    /* num_valuators: */  1,
+			    absX);
+    } else if (*absYset) {
+	xf86PostMotionEvent(pInfo->dev, 
+			    /* is_absolute: */    TRUE,
+			    /* first_valuator: */ 1,
+			    /* num_valuators: */  1,
+			    absY);
+    }
+
+    *absXset = FALSE;
+    *absYset = FALSE;
+}
+
 static void
 vuidReadInput(InputInfoPtr pInfo)
 {
@@ -234,9 +324,11 @@ vuidReadInput(InputInfoPtr pInfo)
     int c; 
     unsigned char *pBuf;
     int wmask;
+    int absX, absY;
+    Bool absXset = FALSE, absYset = FALSE;
 
     pMse = pInfo->private;
-    pVuidMse = pMse->mousePriv;
+    pVuidMse = getVuidMsePriv(pInfo);
     buttons = pMse->lastButtons;
     XisbBlockDuration(pMse->buffer, -1);
     pBuf = pVuidMse->buffer;
@@ -255,9 +347,15 @@ vuidReadInput(InputInfoPtr pInfo)
 			pInfo->name, n);
 	}
 
+#ifdef DEBUG
+	ErrorF("vuidReadInput: event type: %3d value: %5d\n",
+	       pVuidMse->event.id, pVuidMse->event.value);
+#endif
+
 	if (pVuidMse->event.id >= BUT_FIRST && pVuidMse->event.id <= BUT_LAST) {
 	    /* button */
 	    int butnum = pVuidMse->event.id - BUT_FIRST;
+
 	    if (butnum < 3)
 		butnum = 2 - butnum;
 	    if (!pVuidMse->event.value)
@@ -275,6 +373,20 @@ vuidReadInput(InputInfoPtr pInfo)
 	    case LOC_Y_DELTA:
 		dy -= delta;
 		break;
+	    case LOC_X_ABSOLUTE:
+		if (absXset) {
+		    vuidFlushAbsEvents(pInfo, absX, absY, &absXset, &absYset);
+		}
+		absX = delta;
+		absXset = TRUE;
+		break;
+	    case LOC_Y_ABSOLUTE:
+		if (absYset) {
+		    vuidFlushAbsEvents(pInfo, absX, absY, &absXset, &absYset);
+		}
+		absY = delta;
+		absYset = TRUE;
+		break;
 	    }
 	} 
 #ifdef HAVE_VUID_WHEEL
@@ -285,6 +397,13 @@ vuidReadInput(InputInfoPtr pInfo)
 		dw -= VUID_WHEEL_GETDELTA(pVuidMse->event.value);
 	}
 #endif
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING
+	else if (pVuidMse->event.id == MOUSE_TYPE_ABSOLUTE) {
+	    /* force sending absolute resolution scaling ioctl */
+	    pVuidMse->absres.height = pVuidMse->absres.width = 0;
+	    vuidMouseSendScreenSize(miPointerCurrentScreen(), pVuidMse);
+	}
+#endif
 
 	n = 0;
 	if ((c = XisbRead(pMse->buffer)) >= 0) {
@@ -293,11 +412,68 @@ vuidReadInput(InputInfoPtr pInfo)
 	}
     } while (n != 0);
 
+    if (absXset || absYset) {
+	vuidFlushAbsEvents(pInfo, absX, absY, &absXset, &absYset);
+    }
+
     pMse->PostEvent(pInfo, buttons, dx, dy, dz, dw);
     return;
 }
 
-#define NUMEVENTS 64
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING
+static void vuidMouseSendScreenSize(ScreenPtr pScreen, VuidMsePtr pVuidMse)
+{
+    InputInfoPtr pInfo = pVuidMse->pInfo;
+    ScrnInfoPtr pScr = XF86SCRNINFO(pScreen);
+    int result;
+
+    if ((pVuidMse->absres.width != pScr->currentMode->HDisplay) || 
+	(pVuidMse->absres.height != pScr->currentMode->VDisplay))
+    {
+	pVuidMse->absres.width = pScr->currentMode->HDisplay;
+	pVuidMse->absres.height = pScr->currentMode->VDisplay;
+
+	do {
+	    result = ioctl(pInfo->fd, MSIOSRESOLUTION, &(pVuidMse->absres));
+	} while ( (result != 0) && (errno == EINTR) );
+
+	if (result != 0) {
+	    xf86Msg(X_WARNING, 
+		    "%s: couldn't set absolute mouse scaling resolution: %s\n",
+		    pInfo->name, strerror(errno));
+#ifdef DEBUG
+	} else {
+	    xf86Msg(X_INFO, 
+		    "%s: absolute mouse scaling resolution set to %d x %d\n", 
+		    pInfo->name, 
+		    pVuidMse->absres.width, pVuidMse->absres.height);
+#endif
+	}
+    }
+}
+
+static void vuidMouseAdjustFrame(int index, int x, int y, int flags)
+{
+      ScrnInfoPtr	pScrn = xf86Screens[index];
+      ScreenPtr		pScreen = pScrn->pScreen;
+      xf86AdjustFrameProc *wrappedAdjustFrame 
+	  = (xf86AdjustFrameProc *) vuidMouseScreenPrivate(pScreen);
+      VuidMsePtr	m;
+
+      if(wrappedAdjustFrame) {
+        pScrn->AdjustFrame = wrappedAdjustFrame;
+        (*pScrn->AdjustFrame)(index, x, y, flags);
+        pScrn->AdjustFrame = vuidMouseAdjustFrame;
+      }
+
+      if (miPointerCurrentScreen() == pScreen) {
+	  for (m = vuidMouseList; m != NULL ; m = m->next) {
+	      vuidMouseSendScreenSize(pScreen, m);
+	  }
+      }
+}
+#endif /* HAVE_ABSOLUTE_MOUSE_SCALING */
+
 
 static int
 vuidMouseProc(DeviceIntPtr pPointer, int what)
@@ -305,93 +481,88 @@ vuidMouseProc(DeviceIntPtr pPointer, int what)
     InputInfoPtr pInfo;
     MouseDevPtr pMse;
     VuidMsePtr pVuidMse;
-    unsigned char map[MSE_MAXBUTTONS + 1];
-    int nbuttons;
+    int ret = Success;
+    int i;
 
     pInfo = pPointer->public.devicePrivate;
     pMse = pInfo->private;
     pMse->device = pPointer;
-    pVuidMse = pMse->mousePriv;
 
+    pVuidMse = getVuidMsePriv(pInfo);
+    if (pVuidMse == NULL) {
+	return BadImplementation;
+    }
+    
     switch (what) {
-    case DEVICE_INIT: 
-	pPointer->public.on = FALSE;
 
-	for (nbuttons = 0; nbuttons < MSE_MAXBUTTONS; ++nbuttons)
-	    map[nbuttons + 1] = nbuttons + 1;
-
-	InitPointerDeviceStruct((DevicePtr)pPointer, 
-				map, 
-				min(pMse->buttons, MSE_MAXBUTTONS),
-				miPointerGetMotionEvents, 
-				pMse->Ctrl,
-				miPointerGetMotionBufferSize());
-
-	/* X valuator */
-	xf86InitValuatorAxisStruct(pPointer, 0, 0, -1, 1, 0, 1);
-	xf86InitValuatorDefaults(pPointer, 0);
-	/* Y valuator */
-	xf86InitValuatorAxisStruct(pPointer, 1, 0, -1, 1, 0, 1);
-	xf86InitValuatorDefaults(pPointer, 1);
-	xf86MotionHistoryAllocate(pInfo);
-	break;
-
-    case DEVICE_ON:
-	pInfo->fd = xf86OpenSerial(pInfo->options);
-	if (pInfo->fd == -1)
-	    xf86Msg(X_WARNING, "%s: cannot open input device\n", pInfo->name);
-	else {
-	    pMse->buffer = XisbNew(pInfo->fd,
-			      NUMEVENTS * sizeof(Firm_event));
-	    if (!pMse->buffer) {
-		xfree(pMse);
-		xf86CloseSerial(pInfo->fd);
-		pInfo->fd = -1;
-	    } else {
-	        int fmt = VUID_FIRM_EVENT;
-
-		if (pVuidMse->strmod && 
-		    (ioctl(pInfo->fd, I_PUSH, pVuidMse->strmod) == -1)) {
-		    xf86Msg(X_ERROR,
-		      "%s: cannot push module '%s' onto mouse device: %s\n",
-		      pInfo->name, pVuidMse->strmod, strerror(errno));
-		    xf86CloseSerial(pInfo->fd);
-		    pInfo->fd = -1;
-		} else {
-		    ioctl(pInfo->fd, VUIDSFORMAT, &fmt);
-		    vuidMouseWheelInit(pInfo);
-		    xf86FlushInput(pInfo->fd);
-		    AddEnabledDevice(pInfo->fd);
+    case DEVICE_INIT:
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING
+	if (vuidMouseGeneration != serverGeneration) {
+	    if ((vuidMouseScreenIndex = AllocateScreenPrivateIndex()) >= 0) {
+		for (i = 0; i < screenInfo.numScreens; i++) {
+		    ScreenPtr pScreen = screenInfo.screens[i];
+		    ScrnInfoPtr pScrn = XF86SCRNINFO(pScreen);
+		    vuidMouseScreenPrivate(pScreen)
+			= (pointer) pScrn->AdjustFrame;
+		    pScrn->AdjustFrame = vuidMouseAdjustFrame;
 		}
 	    }
+	    vuidMouseGeneration = serverGeneration;
+	}
+#endif    	
+	ret = pVuidMse->wrapped_device_control(pPointer, what);
+	break;
+	
+    case DEVICE_ON:
+	ret = pVuidMse->wrapped_device_control(pPointer, DEVICE_ON);
+
+	if ((ret == Success) && (pInfo->fd != -1)) {
+	    int fmt = VUID_FIRM_EVENT;
+	    
+	    if (pVuidMse->strmod) {
+		SYSCALL(i = ioctl(pInfo->fd, I_PUSH, pVuidMse->strmod));
+		if (i < 0) {
+		    xf86Msg(X_WARNING,
+			"%s: cannot push module '%s' onto mouse device: %s\n",
+			pInfo->name, pVuidMse->strmod, strerror(errno));
+		    xfree(pVuidMse->strmod);
+		    pVuidMse->strmod = NULL;
+		}
+	    }
+	    SYSCALL(i = ioctl(pInfo->fd, VUIDSFORMAT, &fmt));
+	    if (i < 0) {
+		xf86Msg(X_WARNING,
+			"%s: cannot set mouse device to VUID mode: %s\n",
+			pInfo->name, strerror(errno));
+	    }
+	    vuidMouseWheelInit(pInfo);
+#ifdef HAVE_ABSOLUTE_MOUSE_SCALING	    
+	    vuidMouseSendScreenSize(screenInfo.screens[0], pVuidMse);
+#endif	    
+	    xf86FlushInput(pInfo->fd);
 	}
-	pMse->lastButtons = 0;
-	pMse->emulateState = 0;
-	pPointer->public.on = TRUE;
 	break;
 
     case DEVICE_OFF:
     case DEVICE_CLOSE:
 	if (pInfo->fd != -1) {
-	    RemoveEnabledDevice(pInfo->fd);
-	    if (pMse->buffer) {
-		XisbFree(pMse->buffer);
-		pMse->buffer = NULL;
-	    }
-	    if (pVuidMse->strmod && 
-		(ioctl(pInfo->fd, I_POP, pVuidMse->strmod) == -1)) {
-		xf86Msg(X_WARNING,
+	    if (pVuidMse->strmod) {
+		SYSCALL(i = ioctl(pInfo->fd, I_POP, pVuidMse->strmod));
+		if (i == -1) {
+		    xf86Msg(X_WARNING,
 		      "%s: cannot pop module '%s' off mouse device: %s\n",
 		      pInfo->name, pVuidMse->strmod, strerror(errno));
+		}
 	    }
-	    xf86CloseSerial(pInfo->fd);
-	    pInfo->fd = -1;
 	}
-	pPointer->public.on = FALSE;
-	usleep(300000);
+	ret = pVuidMse->wrapped_device_control(pPointer, what);
+	break;
+
+    default: /* Should never be called, but just in case */
+	ret = pVuidMse->wrapped_device_control(pPointer, what);
 	break;
     }
-    return Success;
+    return ret;
 }
 
 static Bool
@@ -453,8 +624,9 @@ solarisMouseAutoProbe(InputInfoPtr pInfo, const char **protocol,
 	} else {
 	    found = TRUE;
 	    if ((*pproto != NULL) && (strcmp(*pproto, "VUID") == 0)) {
-		int i;
-		if (ioctl(fd, VUIDGFORMAT, &i) < 0) {
+		int i, r;
+		SYSCALL(r = ioctl(fd, VUIDGFORMAT, &i));
+    		if (r < 0) {
 		    found = FALSE;
 		}
 	    }
